from datetime import datetime
from pydantic import BaseModel
from pydantic import Field
from typing import List
from typing import Optional
from uuid import uuid4, UUID
from typing import TYPE_CHECKING
from arend.brokers.beanstalkd import BeanstalkdConnection
from arend.settings.tasks import (
    TASK_DELAY,
    TASK_PRIORITY,
    TASK_RETRY_BACKOFF_FACTOR,
    TASK_MAX_RETRIES,
)

import importlib
import logging
import traceback


if TYPE_CHECKING:
    from arend.arend import ArendTask
    from arend.settings.arend import ArendSettings


logger = logging.getLogger(__name__)


class Status:
    SCHEDULED: str = "SCHEDULED"
    PENDING: str = "PENDING"
    STARTED: str = "STARTED"
    RETRY: str = "RETRY"
    FINISHED: str = "FINISHED"
    FAIL: str = "FAIL"
    REVOKED: str = "REVOKED"


class BaseTask(BaseModel):
    """
    Base Task
    """

    uuid: UUID = Field(default_factory=uuid4, description="UUID")
    name: str = Field(..., description="Full path task name")
    description: str = Field(default=None, description="Description")
    location: str = Field(default="tasks", description="Module location")

    status: str = Field(default=Status.SCHEDULED, description="Status")
    result: str = Field(default=None, description="Task result")
    detail: Optional[str] = Field(description="Task details")
    start_time: Optional[datetime] = Field(default=None)
    end_time: Optional[datetime] = Field(default=None)

    args: tuple = Field(default_factory=tuple, description="Task args")
    kwargs: dict = Field(default_factory=dict, description="Task kwargs")

    queue: str = Field(..., description="Queue name")
    delay: int = Field(default=TASK_DELAY, description="Queue delay")
    priority: int = Field(default=TASK_PRIORITY, description="Queue priority")
    count_retries: int = Field(default=0, description="Number of retries")
    max_retries: int = Field(default=TASK_MAX_RETRIES)
    retry_backoff_factor: int = Field(default=TASK_RETRY_BACKOFF_FACTOR)

    created: datetime = Field(default_factory=datetime.utcnow)
    updated: datetime = Field(default=None)

    class Meta:
        settings: "ArendSettings"

    def save(self):
        return NotImplementedError

    def delete(self):
        return NotImplementedError

    @classmethod
    def get(cls, uuid: UUID):
        return NotImplementedError

    def send_to_queue(self):
        with BeanstalkdConnection(
            queue=self.queue, settings=self.Meta.settings.beanstalkd
        ) as conn:
            conn.put(
                body=str(self.uuid),
                priority=self.priority,
                delay=self.delay
                + self.count_retries * self.retry_backoff_factor,
            )
            self.status = Status.PENDING
            self.save()

    def __enter__(self):
        if self.status == [Status.REVOKED, Status.FAIL, Status.FINISHED]:
            return

        self.start_time = datetime.utcnow()
        self.status = Status.STARTED
        self.save()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type:
            last_trace = "".join(traceback.format_tb(exc_tb)).strip()
            self.detail = f"Failure: {last_trace}\n"
            if self.count_retries < self.max_retries:
                self.count_retries += 1
                self.status = Status.RETRY
                self.send_to_queue()  # put it in the tube again
            else:
                self.status = Status.FAIL
        else:
            self.status = Status.FINISHED

        self.end_time = datetime.utcnow()
        self.save()
        return True

    def __call__(self):
        return self.run()

    def run(self):
        """
        Run task.
        - Get signature
        - Run signature with args and kwargs
        - Update result
        """
        with self:
            task = self.get_task_signature()
            result = task.run(*self.args, **self.kwargs)
            self.result = result

    def get_task_signature(self) -> "ArendTask":
        """
        Get task signature.

        Returns: ArendTask.
        """
        module = importlib.import_module(self.location)
        task = getattr(module, self.name)
        return task


class BaseTasks(BaseModel):
    tasks: List[BaseTask] = Field(default_factory=list)
    count: int = Field(default=0)



from pydantic import BaseSettings, BaseModel
from typing import Union, Type
from arend.brokers.beanstalkd import BeanstalkdSettings
from arend.backends.redis import RedisSettings, RedisTask
from arend.backends.mongo import MongoSettings, MongoTask
from arend.settings.tasks import (
    TASK_DELAY,
    TASK_PRIORITY,
    TASK_RETRY_BACKOFF_FACTOR,
    TASK_MAX_RETRIES,
)

import logging


logger = logging.getLogger(__name__)


__all__ = [
    "Settings",
    "ArendSettings",
    "MongoSettings",
    "RedisSettings",
    "RedisTask",
    "MongoTask",
]


class ArendSettings(BaseModel):
    """
    Defines settings for the Arend

    Usage:
        >>> from arend import arend_task
        >>> from arend.backends.mongo import MongoSettings
        >>> from arend.brokers import BeanstalkdSettings
        >>> from arend.settings import ArendSettings
        >>> from arend.worker import consumer
        >>>
        >>> settings = ArendSettings(
        >>>     beanstalkd=BeanstalkdSettings(host="beanstalkd", port=11300),
        >>>     backend=MongoSettings(
        >>>         mongo_connection="mongodb://user:pass@mongo:27017",
        >>>         mongo_db="db",
        >>>         mongo_collection="Tasks"
        >>>     ),
        >>>     task_max_retries = 3
        >>>     task_retry_backoff_factor = 1
        >>>     task_priority = 0
        >>>     task_delay = 1
        >>> )
    """

    beanstalkd: BeanstalkdSettings
    backend: Union[MongoSettings, RedisSettings]
    task_max_retries: int = TASK_MAX_RETRIES
    task_retry_backoff_factor: int = TASK_RETRY_BACKOFF_FACTOR
    task_priority: int = TASK_PRIORITY
    task_delay: int = TASK_DELAY

    def get_backend(self) -> Type[Union[MongoTask, RedisTask]]:
        """
        Return a Task Backend with configuration already set
        """
        Task = self.backend.get_backend()
        Task.Meta.settings = self
        return Task


class Settings(BaseSettings):

    arend: ArendSettings

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"
        env_nested_delimiter = "__"



import logging


logger = logging.getLogger(__name__)


__all__ = [
    "TASK_MAX_RETRIES",
    "TASK_DELAY",
    "TASK_PRIORITY",
    "TASK_DELAY_FACTOR",
    "TASK_RETRY_BACKOFF_FACTOR",
]

# settings
TASK_MAX_RETRIES = 3
TASK_RETRY_BACKOFF_FACTOR = 1
TASK_PRIORITY = 1
TASK_DELAY_FACTOR: int = 1
TASK_DELAY: int = 0



from pystalkd.Beanstalkd import Connection
from pydantic import BaseModel
from pydantic import Field

import logging

__all__ = ["BeanstalkdConnection", "BeanstalkdSettings"]


logger = logging.getLogger(__name__)


class BeanstalkdSettings(BaseModel):
    """
    Defines settings for the Beanstalkd Queue
    """

    host: str = Field(description="Beanstalkd Host")
    port: int = Field(description="Beanstalkd Port")


class BeanstalkdConnection:
    """
    Beanstalkd Connection.

    Defines a context manager to open and close connection
    when interacting with our queue.

    Usage:
        >>> from arend.brokers import BeanstalkdConnection, BeanstalkdSettings
        >>> from arend.backends.mongo import MongoSettings
        >>>
        >>> settings = BeanstalkdSettings(host="beanstalkd", port=11300)
        >>> with BeanstalkdConnection(
        >>>     queue="my_queue",
        >>>     settings=settings
        >>> ) as conn:
        >>>     conn.put(body="my message")
        >>>     ...
        >>>
    """

    def __init__(self, queue: str, settings: BeanstalkdSettings):
        self.settings = settings
        self.connection = Connection(
            host=self.settings.host, port=self.settings.port
        )
        self.connection.watch(queue)
        self.connection.use(queue)

    def __enter__(self: "BeanstalkdConnection") -> Connection:
        return self.connection

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.connection.close()



from arend.brokers import BeanstalkdConnection
from arend.settings import Settings, ArendSettings
from uuid import UUID
import logging
import time

__all__ = ["consumer"]


logger = logging.getLogger(__name__)


def consumer(
    queue: str,
    timeout: int = 20,
    sleep_time: float = 0.1,
    long_polling: bool = False,
    settings: ArendSettings = None,
):
    """
    Consumer. Consume tasks from the queue.

    Args:
        queue: str. Queue name.
        timeout: int. Polling timeout.
        sleep_time: float. Sleeping time between polling cycles.
        long_polling: bool. Break the loop if no more messages.
        settings: ArendSettings, None. If no settings are passed,
            the consumer will try to get them from env variables.

    Usage:
        >>> from arend.worker.consumer import consumer
        >>>
        >>> consumer(queue="my_queue", timeout=0)
    """

    settings = settings or Settings().arend
    Task = settings.get_backend()

    while True:

        with BeanstalkdConnection(queue, settings=settings.beanstalkd) as conn:

            message = conn.reserve(timeout=timeout)
            if message is None and not long_polling:
                break  # if not long_polling, consume all messages and break

            if message:
                if task := Task.get(uuid=UUID(message.body)):
                    task.run()  # run task here

                message.delete()

        time.sleep(sleep_time)



import logging
from datetime import datetime
from uuid import UUID
from typing import Type, List, Union, TYPE_CHECKING
from pydantic import BaseModel, Field
from pymongo.collection import Collection
from contextlib import contextmanager
from arend.backends.base import BaseTask
from pymongo.mongo_client import MongoClient


if TYPE_CHECKING:
    from arend.settings import ArendSettings


__all__ = ["MongoTask", "MongoTasks", "MongoSettings"]


logger = logging.getLogger(__name__)


class MongoTask(BaseTask):
    """
    MongoTask class. Defines the Task Model for Mongo Backend

    Usage:

        >>> from arend.backends.mongo import MongoSettings
        >>>
        >>> settings = MongoSettings(
        >>>     mongo_connection="mongodb://user:pass@mongo:27017",
        >>>     mongo_db="db",
        >>>     mongo_collection="Tasks"
        >>> )
        >>> MongoTask = MongoSettings.get_backend()  # type: Type[MongoTask]
        >>> task = MongoTask(name="My task", ...)
        >>> task.save()
        >>>
        >>> assert task.dict() == {"name": "My task", ...}
        >>> assert task.json() == '{"name": "My task", ...}'
        >>>
        >>> task = MongoTask.get(uuid=UUID("<your-uuid>"))
        >>> assert task.description == "A cool task"
        >>>
        >>> assert task.delete() == 1
        >>> assert task.delete() == 0
    """

    class Meta:
        settings: "ArendSettings"

    @classmethod
    @contextmanager
    def mongo_collection(cls):
        """
        Yield a MongoTask Collection
        """
        mongo_conn = cls.Meta.settings.backend.mongo_connection
        mongo_db = cls.Meta.settings.backend.mongo_db
        mongo_collection = cls.Meta.settings.backend.mongo_collection

        with MongoClient(
            mongo_conn, UuidRepresentation="standard"
        ) as client:  # type: MongoClient
            db = client.get_database(mongo_db)
            collection = db.get_collection(mongo_collection)
            yield collection

    @classmethod
    def get(cls, uuid: UUID) -> Union["MongoTask", None]:
        """
        Get object from DataBase

        Usage:

            >>> task = MongoTask.get(uuid=UUID("<your-uuid>"))
            >>> assert task.uuid == UUID("<your-uuid>")
        """
        with cls.mongo_collection() as collection:  # type: Collection
            if task := collection.find_one({"uuid": uuid}):
                return cls(**task)
            return None

    def save(self) -> "MongoTask":
        """
        Updates/Creates object in DataBase

        Usage:

            >>> task = MongoTask(name="My Task")
            >>> task.save()
            >>> task.description = "A new description"
            >>> task.save()
        """
        self.updated = datetime.utcnow()
        with self.mongo_collection() as collection:  # type: Collection
            collection.update_one(
                filter={"uuid": self.uuid},
                update={"$set": self.dict()},
                upsert=True,
            )
        return self

    def delete(self) -> int:
        """
        Deletes object in DataBase

        Usage:

            >>> assert task.delete() == 1  # count deleted 1
            >>> assert task.delete() == 0  # count deleted 0
        """
        with self.mongo_collection() as collection:  # type: Collection
            deleted = collection.delete_one({"uuid": self.uuid})
            return deleted.deleted_count


class MongoTasks(BaseModel):
    """
    Defines the MongoTasks collection
    """

    tasks: List[MongoTask] = Field(default_factory=list, description="Tasks")
    count: int = Field(default=0, description="Count")


class MongoSettings(BaseModel):
    """
    Mongo Settings. Defines settings for Mongo Backend
    """

    mongo_connection: str = Field(..., description="Connection string")
    mongo_db: str = Field(..., description="Database name")
    mongo_collection: str = Field(..., description="Collection name")

    def get_backend(self) -> Type[MongoTask]:
        """
        Returns a MongoTask class with settings already set
        """
        return MongoTask



from arend.arend import ArendTask, arend_task
from arend.worker.consumer import consumer
from arend.worker.pool_consumers import pool_consumers


__all__ = ["ArendTask", "arend_task", "consumer", "pool_consumers"]



from arend.backends.redis import RedisSettings, RedisTask
from arend.backends.mongo import MongoSettings, MongoTask

__all__ = [
    "MongoSettings",
    "RedisSettings",
    "RedisTask",
    "MongoTask",
]



import logging
from redis import Redis  # type: ignore
from typing import Dict, Type, List, Union, TYPE_CHECKING
from datetime import datetime
from uuid import UUID
from pydantic import BaseModel, Field
from arend.backends.base import BaseTask
from contextlib import contextmanager

if TYPE_CHECKING:
    from arend.settings import ArendSettings


__all__ = ["RedisTask", "RedisTasks", "RedisSettings"]


logger = logging.getLogger(__name__)


class RedisTask(BaseTask):
    """
    RedisLog class. Defines the Log for Redis Backend

    Usage:

        >>> from arend.backends.redis import RedisSettings
        >>>
        >>> settings = RedisSettings(
        >>>     redis_host="redis",
        >>>     redis_port=6379,
        >>>     redis_db="logs",
        >>>     redis_password="pass"
        >>> )
        >>> RedisTask = RedisSettings.get_backend()  # type: Type[RedisLog]
        >>> task = RedisTask(name="My task", description="A cool task")
        >>> task.save()
        >>>
        >>> assert task.dict() == {"name": "My task", ...}
        >>> assert task.json() == '{"name": "My task", ...}'
        >>>
        >>> task = RedisTask.get(uuid=UUID("<your-uuid>"))
        >>> assert task.description == "A cool task"
        >>>
        >>> assert task.delete() == 1
    """

    class Meta:
        settings: "ArendSettings"

    @classmethod
    @contextmanager
    def redis_connection(cls):
        """
        Yield a Redis connection to a specific database
        """
        host = cls.Meta.settings.backend.redis_host
        port = cls.Meta.settings.backend.redis_port
        db = cls.Meta.settings.backend.redis_db
        password = cls.Meta.settings.backend.redis_password

        with Redis(host=host, port=port, db=db, password=password) as r:
            yield r

    @classmethod
    def get(cls, uuid: UUID) -> Union["RedisTask", None]:
        """
        Get object from DataBase

        Usage:

            >>> log = RedisTask.get(uuid=UUID("<your-uuid>"))
            >>> assert log.uuid == UUID("<your-uuid>")
        """
        with cls.redis_connection() as r:  # type: Redis
            if task := r.get(str(uuid)):
                return cls.parse_raw(task)
            return None

    def save(self) -> "RedisTask":
        """
        Updates/Creates object in DataBase

        Usage:

            >>> task = RedisTask(name="My Task")
            >>> task.save()
            >>> task.description = "A new description"
            >>> task.save()
        """
        self.updated = datetime.utcnow()
        with self.redis_connection() as r:  # type: Redis
            r.set(str(self.uuid), self.json())
        return self

    def delete(self) -> int:
        """
        Deletes object in DataBase

        Usage:

            >>> assert task.delete() == 1  # count deleted 1
            >>> assert task.delete() == 0  # count deleted 0
        """
        with self.redis_connection() as r:  # type: Redis
            return r.delete(str(self.uuid))


class RedisTasks(BaseModel):
    """
    Defines the RedisTasks collection
    """

    tasks: List[RedisTask] = Field(default_factory=list, description="Logs")
    count: int = Field(default=0, description="Count")


class RedisSettings(BaseModel):
    """
    Defines settings for Redis Backend
    """

    redis_host: str = Field(..., description="Redis Host")
    redis_port: int = Field(default=6379, description="Redis Host")
    redis_db: int = Field(default=1, description="Redis DB")
    redis_password: str = Field(..., description="Redis Password")
    redis_extras: Dict = Field(
        default_factory=dict, description="Redis extras"
    )

    def get_backend(self) -> Type[RedisTask]:
        """
        Returns a RedisTask class with settings already set
        """
        return RedisTask



from arend.worker.consumer import consumer
from multiprocessing import Pool

import click
import logging


__all__ = ["pool_consumers"]


logger = logging.getLogger(__name__)


@click.command(name="Start Pool of Consumers")
@click.argument("args", nargs=-1)
def pool_consumers(args):
    """
    Example:
    ```python3 pool_processor --queue_1=4 --queue_2=2```
    """

    # parse arguments
    queues = []
    for arg in args:
        queue, concurrency = arg.split("=")
        queues.extend([queue] * int(concurrency))

    # get the pool and map the consumers
    with Pool(processes=len(queues)) as pool:
        pool.map(consumer, queues)



from arend.settings import ArendSettings, Settings
from arend.backends import MongoTask, RedisTask
from typing import Callable, Union
from pydantic import BaseModel
from datetime import timedelta
from arend.settings.tasks import (
    TASK_DELAY,
    TASK_PRIORITY,
    TASK_RETRY_BACKOFF_FACTOR,
    TASK_MAX_RETRIES,
)

import functools
import logging

__all__ = ["arend_task", "ArendTask"]


logger = logging.getLogger(__name__)


class ArendTask(BaseModel):
    """
    Defines the ArendTask. This class handles writing
    to the Backend and the Queue the Arend tasks
    """

    queue: str
    name: str
    location: str
    processor: Callable
    priority: int = TASK_PRIORITY
    max_retries: int = TASK_MAX_RETRIES
    delay: Union[timedelta, int] = TASK_DELAY
    retry_backoff_factor: int = TASK_RETRY_BACKOFF_FACTOR
    settings: ArendSettings = None

    def __call__(self, *args, **kwargs):
        return self.run(*args, **kwargs)

    def __repr__(self):
        return f"<{self.__class__.__name__} at {self.__class__.__module__}>"

    def run(self, *args, **kwargs):
        """
        Run the task immediately.
        """
        return self.processor(*args, **kwargs)

    def apply_async(
        self,
        queue: str = None,
        args: tuple = None,
        kwargs: dict = None,
        priority: int = TASK_PRIORITY,
        max_retries: int = TASK_MAX_RETRIES,
        delay: Union[timedelta, int] = TASK_DELAY,
        retry_backoff_factor: int = TASK_RETRY_BACKOFF_FACTOR,
        settings: ArendSettings = settings,
    ) -> Union[MongoTask, RedisTask]:
        """
        Run task asynchronously.
        """
        settings = settings or Settings().arend
        Task = settings.get_backend()

        # create and save a task in your backend
        task = Task(
            name=self.name,
            location=self.location,
            queue=self.queue or queue,
            args=args or (),
            kwargs=kwargs or {},
            priority=priority
            or self.priority
            or settings.task_priority
            or TASK_PRIORITY,
            delay=delay or self.delay or settings.task_delay or TASK_DELAY,
            max_retries=max_retries
            or self.max_retries
            or settings.task_max_retries
            or TASK_MAX_RETRIES,
            retry_backoff_factor=retry_backoff_factor
            or self.retry_backoff_factor
            or settings.task_retry_backoff_factor
            or TASK_RETRY_BACKOFF_FACTOR,
        ).save()  # default to SCHEDULED

        task.send_to_queue()  # put into the queue and set task as PENDING

        return task  # return a PENDING task


def arend_task(
    queue: str = None,
    priority: int = TASK_PRIORITY,
    delay: Union[timedelta, int] = TASK_DELAY,
    max_retries: int = TASK_MAX_RETRIES,
    retry_backoff_factor: int = TASK_RETRY_BACKOFF_FACTOR,
    settings: ArendSettings = None,
):
    """
    Register functions as arend task

    Usage:
        >>> from arend import arend_task
        >>> from arend.backends.mongo import MongoSettings
        >>> from arend.brokers import BeanstalkdSettings
        >>> from arend.settings import ArendSettings
        >>> from arend.worker import consumer
        >>>
        >>> settings = ArendSettings(
        >>>     beanstalkd=BeanstalkdSettings(host="beanstalkd", port=11300),
        >>>     backend=MongoSettings(
        >>>         mongo_connection="mongodb://user:pass@mongo:27017",
        >>>         mongo_db="db",
        >>>         mongo_collection="Tasks"
        >>>     ),
        >>> )
        >>>
        >>> @arend_task(queue="my_queue", settings=settings)
        >>> def double(num: int):
        >>>     return 2 * num
        >>>
        >>> double(2)  # returns 4
        >>> task = double.apply_async(args=(4,))  # It is sent to the queue
        >>>
        >>> consumer(queue="my_queue", settings=settings)  # consume tasks
        >>>
        >>> Task = settings.get_backend()  # you can check your result backend
        >>> task = Task.get(uuid=task.uuid)
        >>> assert task.result == 4

    By defining the backend in the environment variables:
        >>> from arend import arend_task
        >>> from arend.worker import consumer
        >>>
        >>>
        >>> @arend_task(queue="my_queue")
        >>> def double(num: int):
        >>>     return 2 * num
        >>>
        >>> double.apply_async(args=(4,))  # It is sent to the queue
        >>>
        >>> consumer(queue="my_queue")  # consume tasks from the queue
    """

    def decorator(func):
        @functools.wraps(func)
        def wrapper_register():
            return ArendTask(
                name=func.__name__,
                location=func.__module__,
                processor=func,
                queue=queue,
                priority=priority,
                delay=delay,
                max_retries=max_retries,
                retry_backoff_factor=retry_backoff_factor,
                settings=settings,
            )

        return wrapper_register()

    return decorator



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['arend', 'arend.backends', 'arend.brokers', 'arend.settings', 'arend.worker']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'pandas>=1.4.3,<2.0.0',
 'pydantic>=1.9.1,<2.0.0',
 'pymongo>=4.2.0,<5.0.0',
 'pystalkd>=1.3.0,<2.0.0',
 'redis>=4.3.4,<5.0.0']

setup_kwargs = {
    'name': 'arend',
    'version': '0.1.0',
    'description': 'A simple producer consumer library for Beanstalkd.',
    'long_description': 'Arend\n========\n\nA simple producer-consumer library for the Beanstalkd queue.\n\nInstallation\n--------------\nHit the command:\n```shell\npip install arend\n```\n\nBasic Usage\n--------------\n\nIn your code:\n ```python\nfrom arend import arend_task\nfrom arend.backends.mongo import MongoSettings\nfrom arend.brokers import BeanstalkdSettings\nfrom arend.settings import ArendSettings\nfrom arend.worker import consumer\n\nsettings = ArendSettings(\n    beanstalkd=BeanstalkdSettings(host="beanstalkd", port=11300),\n    backend=MongoSettings(\n        mongo_connection="mongodb://user:pass@mongo:27017",\n        mongo_db="db",\n        mongo_collection="Tasks"\n    ),\n)\n\n@arend_task(queue="my_queue", settings=settings)\ndef double(num: int):\n    return 2 * num\n\ndouble(2)  # returns 4\ntask = double.apply_async(args=(4,))  # It is sent to the queue\n\nconsumer(queue="my_queue", settings=settings)  # consume tasks from the queue\n\nTask = settings.get_backend()  # you can check your backend for the result\ntask = Task.get(uuid=task.uuid)\nassert task.result == 4\n```\n\nBackends\n-------------------\nThe available backends to store logs are **Mongo** and **Redis**.\nPlease read the [docs](https://arend.readthedocs.io/en/latest/) \nfor further information.\n\nSetting your backend with environment variables\n--------------------------------------------------\nYou can set your backend by defining env vars.\nThe `AREND__` prefix indicates that it belongs to the Arend.\n```shell\n# Redis\nAREND__REDIS_HOST=\'redis\'\nAREND__REDIS_DB=\'1\'\nAREND__REDIS_PASSWORD=\'pass\'\n...\n\n# Mongo\nAREND__MONGO_CONNECTION=\'mongodb://user:pass@mongo:27017\'\nAREND__MONGO_DB=\'db\'\nAREND__MONGO_COLLECTION=\'logs\'\n...\n```\n\nIn your code:\n ```python\nfrom arend import arend_task\nfrom arend.worker import consumer\n\n\n@arend_task(queue="my_queue")\ndef double(num: int):\n    return 2 * num\n\ndouble.apply_async(args=(4,))  # It is sent to the queue\n\nconsumer(queue="my_queue")\n```\n\nDocumentation\n--------------\n\nPlease visit this [link](https://arend.readthedocs.io/en/latest/) for documentation.\n',
    'author': 'Jose Vazquez',
    'author_email': 'josevazjim88@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pyprogrammerblog/arend',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
